# coding: utf-8

import representatives_votes.models as models

from rest_framework import serializers


class VoteSerializer(serializers.HyperlinkedModelSerializer):
    """
    Vote serializer
    """
    class Meta:
        model = models.Vote
        fields = (
            'proposal',
            'representative',
            'representative_name',
            'position'
        )


class ProposalSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Proposal
        fields = (
            'dossier',
            'title',
            'description',
            'reference',
            'datetime',
            'kind',
            'total_abstain',
            'total_against',
            'total_for',
            'url',
        )


class ProposalDetailSerializer(ProposalSerializer):
    """ Proposal serializer that includes votes """

    votes = VoteSerializer(many=True)

    class Meta:
        model = models.Proposal
        fields = ProposalSerializer.Meta.fields + ('votes',)


class DocumentSerializer(serializers.HyperlinkedModelSerializer):
    """ Base document serializer """

    class Meta:
        model = models.Document
        fields = (
            'dossier',
            'chamber',
            'kind',
            'link'
        )


class DossierSerializer(serializers.HyperlinkedModelSerializer):
    """ Base dossier serializer """

    class Meta:
        model = models.Dossier
        fields = (
            'title',
            'reference',
            'text',
            'url',
        )


class DossierDetailSerializer(DossierSerializer):
    """
    Dossier serializer that includes proposals and votes.
    """

    proposals = ProposalSerializer(many=True)
    documents = DocumentSerializer(many=True)

    class Meta:
        model = models.Dossier
        field = DossierSerializer.Meta.fields + ('proposals', 'documents')



import copy
import mock
import os
import pytest

from django.core.serializers.json import Deserializer
from django.core.management import call_command
from django.test import TestCase
from representatives_votes.contrib.parltrack import import_dossiers
from representatives_votes.contrib.parltrack import import_votes
from representatives_votes.models import Dossier, Proposal, Vote
import representatives
from representatives.models import Representative


def _test_import(scenario, callback):
    fixture = os.path.join(os.path.dirname(__file__),
            '%s_fixture.json' % scenario)
    expected = os.path.join(os.path.dirname(__file__),
            '%s_expected.json' % scenario)

    # Disable django auto fields
    exclude = ('id', '_state', 'created', 'updated')

    with open(fixture, 'r') as f:
        callback(f)

    with open(expected, 'r') as f:
        for obj in Deserializer(f.read()):
            compare = copy.copy(obj.object.__dict__)

            for f in exclude:
                if f in compare:
                    compare.pop(f)

            type(obj.object).objects.get(**compare)


@pytest.mark.django_db
def test_parltrack_import_dossiers():
    _test_import('dossiers', import_dossiers.main)


@pytest.mark.django_db
def test_parltrack_import_votes():
    for model in (Representative, Dossier, Proposal, Vote):
        model.objects.all().delete()

    call_command('loaddata', os.path.join(os.path.abspath(
        representatives.__path__[0]), 'fixtures', 'representatives_test.json'))
    call_command('loaddata', os.path.join(os.path.dirname(__file__),
        'dossiers_expected.json'))

    _test_import('votes', import_votes.main)


class DossierTest(TestCase):
    def setUp(self):
        for model in (Representative, Dossier, Proposal, Vote):
            model.objects.all().delete()

    def test_parltrack_import_single_dossier(self):
        call_command('loaddata', os.path.join(os.path.abspath(
            representatives.__path__[0]), 'fixtures',
            'representatives_test.json'))

        with self.assertNumQueries(22):
            _test_import('single', import_dossiers.import_single)

    def test_parltrack_sync_dossier(self):
        call_command('loaddata', os.path.join(os.path.abspath(
            representatives.__path__[0]), 'fixtures',
            'representatives_test.json'))
        call_command('loaddata', os.path.join(os.path.dirname(__file__),
            'sync_fixture.json'))

        reference = '2012/2002(INI)'
        mock_file = os.path.join(os.path.dirname(__file__),
            'parltrack_dossier_%s.json' % reference.replace('/', '-'))
        expected_url = '%s/dossier/%s?format=json' % (import_dossiers.BASEURL,
            reference)

        def callback(stream):
            import_dossiers.sync_dossier(reference)

        with mock.patch('urllib2.urlopen') as urlopen:
            with open(mock_file, 'r') as mock_stream:
                urlopen.return_value = mock_stream

                with self.assertNumQueries(8):
                    _test_import('sync', callback)

            urlopen.assert_called_with(expected_url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from django.db import migrations, models


def migrate_dossier_links(apps, schema_editor):
    logger = logging.getLogger('migrate_dossier_links')

    # Get model managers
    Chamber = apps.get_model("representatives", "Chamber")
    Dossier = apps.get_model("representatives_votes", "Dossier")
    Document = apps.get_model("representatives_votes", "Document")

    docs = []

    # EP dossiers
    ep_chamber = Chamber.objects.get(abbreviation='EP')
    ep_link = 'europarl.europa.eu'
    for dossier in Dossier.objects.filter(link__icontains=ep_link):
        logger.info('Create document %s for dossier %s' % (dossier.link,
                                                           dossier.reference))
        docs.append(Document(chamber=ep_chamber, dossier=dossier,
                             link=dossier.link, kind='procedure-file'))

    # France dossiers
    try:
        an_chamber = Chamber.objects.get(abbreviation='AN')
        sen_chamber = Chamber.objects.get(abbreviation='SEN')
    except Chamber.DoesNotExist:
        return

    an_link = 'assemblee-nationale.fr'
    sen_link = 'senat.fr'

    for dossier in Dossier.objects.filter(link__icontains=an_link):
        logger.info('Create document %s for dossier %s' % (dossier.link,
                                                           dossier.reference))
        docs.append(Document(chamber=an_chamber, dossier=dossier,
                             link=dossier.link, kind='procedure-file'))

    for dossier in Dossier.objects.filter(ext_link__icontains=an_link):
        logger.info('Create document %s for dossier %s' % (dossier.link,
                                                           dossier.reference))
        docs.append(Document(chamber=an_chamber, dossier=dossier,
                             link=dossier.ext_link, kind='procedure-file'))

    for dossier in Dossier.objects.filter(link__icontains=sen_link):
        logger.info('Create document %s for dossier %s' % (dossier.link,
                                                           dossier.reference))
        docs.append(Document(chamber=sen_chamber, dossier=dossier,
                             link=dossier.link, kind='procedure-file'))

    for dossier in Dossier.objects.filter(ext_link__icontains=an_link):
        logger.info('Create document %s for dossier %s' % (dossier.link,
                                                           dossier.reference))
        docs.append(Document(chamber=sen_chamber, dossier=dossier,
                             link=dossier.ext_link, kind='procedure-file'))

    # Create all dossiers
    logger.info('Saving %s documents...' % len(docs))
    Document.objects.bulk_create(docs)


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0019_remove_fingerprints'),
        ('representatives_votes', '0011_remove_fingerprints'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=1000)),
                ('kind', models.CharField(default=b'', max_length=255, blank=True)),
                ('link', models.URLField(max_length=1000)),
                ('chamber', models.ForeignKey(to='representatives.Chamber')),
                ('dossier', models.ForeignKey(related_name='documents', to='representatives_votes.Dossier')),
            ],
            options={
                'abstract': False,
            },
        ),

        migrations.RunPython(migrate_dossier_links),

        migrations.RemoveField(
            model_name='dossier',
            name='link',
        ),

        migrations.RemoveField(
            model_name='dossier',
            name='ext_link',
        ),
    ]



# coding: utf-8

from datetime import datetime
import ijson
import logging
from pytz import timezone as date_timezone
import sys

import django
from django.apps import apps
from django.utils.timezone import make_aware as date_make_aware

from representatives_votes.models import Dossier, Proposal

logger = logging.getLogger(__name__)


def _parse_date(date_str):
    return date_make_aware(
        datetime.strptime(date_str, "%Y-%m-%d"),
        date_timezone('Europe/Paris')
    )


def _get_unique_title(proposal_pk, candidate):
    title = candidate

    try:
        exists = Proposal.objects.get(title=title)
    except Proposal.DoesNotExist:
        exists = None

    if exists and exists.pk != proposal_pk:
        num = 1
        while exists and exists.pk != proposal_pk:
            title = '%s (%d)' % (candidate, num)

            try:
                exists = Proposal.objects.get(title=title)
            except Proposal.DoesNotExist:
                exists = None

            num = num + 1

        logger.debug('Made unique title %s' % title)

    return title


class ScrutinImporter:
    dossiers = {}

    def get_dossier(self, url):
        if url not in self.dossiers:
            try:
                self.dossiers[url] = Dossier.objects.get(documents__link=url)
            except Dossier.DoesNotExist:
                return None

        return self.dossiers[url]

    def parse_scrutin_data(self, data):
        ref = data['url']

        if 'dossier_url' not in data:
            logger.debug('Cannot create proposal without dossier')
            return

        dossier = self.get_dossier(data['dossier_url'])
        if dossier is None:
            logger.debug('Cannot create proposal for unknown dossier %s'
                         % data['dossier_url'])
            return

        changed = False
        try:
            proposal = Proposal.objects.get(reference=ref)
        except Proposal.DoesNotExist:
            proposal = Proposal(reference=ref, total_for=0, total_against=0,
                                total_abstain=0)
            logger.debug('Created proposal %s' % ref)
            changed = True

        values = dict(
            title=_get_unique_title(proposal.pk, data["objet"]),
            datetime=_parse_date(data["date"]),
            dossier_id=dossier.pk,
            kind='dossier'
        )

        for key, value in values.items():
            if value != getattr(proposal, key, None):
                logger.debug('Changed proposal %s to %s' % (key, value))
                setattr(proposal, key, value)
                changed = True

        if changed:
            logger.debug('Updated proposal %s' % ref)
            proposal.save()


def main(stream=None):
    if not apps.ready:
        django.setup()

    importer = ScrutinImporter()

    for data in ijson.items(stream or sys.stdin, 'item'):
        importer.parse_scrutin_data(data)



# coding: utf-8
import logging
import re
import sys
from os.path import join

import django.dispatch
import ijson
import django
from django.apps import apps
from dateutil.parser import parse as date_parse
from django.db import transaction
from django.utils.timezone import make_aware as date_make_aware
from pytz import timezone as date_timezone

from representatives.models import Representative
from representatives_votes.models import Dossier, Proposal, Vote


logger = logging.getLogger(__name__)
vote_pre_import = django.dispatch.Signal(providing_args=['vote_data'])


def _parse_date(date_str):
    return date_make_aware(
        date_parse(date_str),
        date_timezone('Europe/Brussels'))

JSON_URL = 'http://parltrack.euwiki.org/dumps/ep_votes.json.xz'
DESTINATION = join('/tmp', 'ep_votes.json')


class Command(object):
    def init_cache(self):
        self.cache = dict()
        self.index_representatives()
        self.index_dossiers()

    def parse_vote_data(self, vote_data):
        """
        Parse data from parltrack votes db dumps (1 proposal)
        """
        if 'epref' not in vote_data.keys():
            logger.debug('Could not import data without epref %s',
                vote_data['title'])
            return

        dossier_pk = self.get_dossier(vote_data['epref'])

        if not dossier_pk:
            logger.debug('Cannot find dossier with remote id %s',
                         vote_data['epref'])
            return

        return self.parse_proposal_data(
            proposal_data=vote_data,
            dossier_pk=dossier_pk
        )

    @transaction.atomic
    def parse_proposal_data(self, proposal_data, dossier_pk):
        """Get or Create a proposal model from raw data"""
        proposal_display = '{} ({})'.format(proposal_data['title'].encode(
            'utf-8'), proposal_data.get('report', '').encode('utf-8'))

        if 'issue_type' not in proposal_data.keys():
            logger.debug('This proposal data without issue_type: %s',
                         proposal_data['epref'])
            return

        changed = False
        try:
            proposal = Proposal.objects.get(title=proposal_data['title'])
        except Proposal.DoesNotExist:
            proposal = Proposal(title=proposal_data['title'])
            changed = True

        data_map = dict(
            title=proposal_data['title'],
            datetime=_parse_date(proposal_data['ts']),
            dossier_id=dossier_pk,
            reference=proposal_data.get('report'),
            kind=proposal_data.get('issue_type')
        )

        for position in ('For', 'Abstain', 'Against'):
            position_data = proposal_data.get(position, {})
            position_total = position_data.get('total', 0)

            if isinstance(position_total, str) and position_total.isdigit():
                position_total = int(position_total)

            data_map['total_%s' % position.lower()] = position_total

        for key, value in data_map.items():
            if value != getattr(proposal, key, None):
                setattr(proposal, key, value)
                changed = True

        if changed:
            proposal.save()

        responses = vote_pre_import.send(sender=self, vote_data=proposal_data)

        for receiver, response in responses:
            if response is False:
                logger.debug(
                    'Skipping dossier %s', proposal_data.get(
                        'epref', proposal_data['title']))
                return

        positions = ['For', 'Abstain', 'Against']
        logger.info(
            'Looking for votes in proposal {}'.format(proposal_display))
        for position in positions:
            for group_vote_data in proposal_data.get(
                    position,
                    {}).get(
                    'groups',
                    {}):
                for vote_data in group_vote_data['votes']:
                    if not isinstance(vote_data, dict):
                        logger.error('Skipping vote data %s for proposal %s',
                                     vote_data, proposal_data['_id'])
                        continue

                    representative_pk = self.get_representative(vote_data)

                    if representative_pk is None:
                        logger.error('Could not find mep for %s', vote_data)
                        continue

                    representative_name = vote_data.get('orig', '')

                    changed = False
                    try:
                        vote = Vote.objects.get(
                            representative_id=representative_pk,
                            proposal_id=proposal.pk)
                    except Vote.DoesNotExist:
                        vote = Vote(proposal_id=proposal.pk,
                            representative_id=representative_pk)
                        changed = True

                    if vote.position != position.lower():
                        changed = True
                        vote.position = position.lower()

                    if vote.representative_name != representative_name:
                        changed = True
                        vote.representative_name = representative_name

                    if changed:
                        vote.save()
                        logger.debug('Save vote %s for MEP %s on %s #%s to %s',
                            vote.pk, representative_pk, proposal_data['title'],
                            proposal.pk, position)

        return proposal

    def index_dossiers(self):
        self.cache['dossiers'] = {
            d[0]: d[1] for d in Dossier.objects.values_list('reference', 'pk')
        }

    def get_dossier(self, reference):
        return self.cache['dossiers'].get(reference, None)

    def index_representatives(self):
        epre = r'/meps/en/(\d+)/_home.html'
        self.cache['meps'] = {
            int(re.search(epre, l[0]).group(1)): l[1] for l in
            Representative.objects.prefetch_related('website_set')
            .filter(website__kind='EP')
            .values_list('website__url', 'pk')
        }

    def get_representative(self, vote_data):
        if vote_data.get('ep_id', None) is None:
            return
        return self.cache['meps'].get(int(vote_data['ep_id']), None)


def main(stream=None):
    if not apps.ready:
        django.setup()

    command = Command()
    command.init_cache()

    for vote_data in ijson.items(stream or sys.stdin, 'item'):
        command.parse_vote_data(vote_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives', '0003_auto_20150702_1827'),
        ('representatives_votes', '0002_auto_20150707_1611'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='vote',
            name='representative_fingerprint',
        ),
        migrations.AddField(
            model_name='proposal',
            name='representatives',
            field=models.ManyToManyField(
                to='representatives.Representative',
                through='representatives_votes.Vote'),
        ),
    ]



# coding: utf-8

import sys
import ijson
import logging
import re

import django
from django.apps import apps
from django.db import transaction

from representatives.contrib.francedata.import_representatives import \
    ensure_chambers
from representatives.models import Chamber
from representatives_votes.models import Document, Dossier

logger = logging.getLogger(__name__)


def extract_reference(url):
    m = re.search(r'/dossier-legislatif/([^./]+)\.html', url)
    if m:
        return m.group(1)

    m = re.search(r'/(\d+)/dossiers/([^./]+)\.asp', url)
    if m:
        return '%s/%s' % (m.group(1), m.group(2))

    m = re.search(r'/dossiers/([^./]+)\.asp', url)
    if m:
        return m.group(1)

    return None


def find_dossier(data):
    '''
    Find dossier with reference matching either 'ref_an' or 'ref_sen',
    create it if not found.  Ensure its reference is 'ref_an' if both fields
    are present.
    '''

    changed = False
    dossier = None
    reffield = None

    for field in [k for k in ('ref_an', 'ref_sen') if k in data]:
        try:
            dossier = Dossier.objects.get(reference=data[field])
            reffield = field
            break
        except Dossier.DoesNotExist:
            pass

    if dossier is None:
        reffield = 'ref_an' if 'ref_an' in data else 'ref_sen'
        dossier = Dossier(reference=data[reffield])
        logger.debug('Created dossier %s' % data[reffield])
        changed = True

    if 'ref_an' in data and reffield != 'ref_an':
        logger.debug('Changed dossier reference to %s' % data['ref_an'])
        dossier.reference = data['ref_an']
        changed = True

    return dossier, changed


def handle_document(dossier, chamber, url):
    doc_changed = False
    try:
        doc = Document.objects.get(chamber=chamber, dossier=dossier,
                                   kind='procedure-file')
    except Document.DoesNotExist:
        doc = Document(chamber=chamber, dossier=dossier, kind='procedure-file')
        logger.debug('Created %s document for dossier %s' %
            (chamber.abbreviation, dossier.title))
        doc_changed = True

    if doc.link != url:
        logger.debug('Changing %s url from %s to %s' %
            (chamber.abbreviation, doc.link, url))
        doc.link = url
        doc_changed = True

    if doc_changed:
        doc.save()


def parse_dossier_data(data, an, sen):
    if 'url_an' in data:
        ref_an = extract_reference(data['url_an'])
        if ref_an is None:
            logger.warn('No reference for dossier %s' % data['url_an'])
            return
        else:
            data['ref_an'] = ref_an

    if 'url_sen' in data:
        ref_sen = extract_reference(data['url_sen'])
        if ref_sen is None:
            logger.warn('No reference for dossier %s' % data['url_sen'])
            return
        else:
            data['ref_sen'] = ref_sen

    dossier, changed = find_dossier(data)

    thisref = data['ref_an' if data['chambre'] == 'AN' else 'ref_sen']

    title = data['titre']
    if dossier.reference == thisref and dossier.title != title:
        logger.debug('Changed dossier title to %s' % title)
        dossier.title = title
        changed = True

    with transaction.atomic():
        if changed:
            logger.debug('Saved dossier %s' % dossier.reference)
            dossier.save()

        if 'url_an' in data:
            handle_document(dossier, an, data['url_an'])

        if 'url_sen' in data:
            handle_document(dossier, sen, data['url_sen'])


def main(stream=None):
    if not apps.ready:
        django.setup()

    ensure_chambers()
    an = Chamber.objects.get(abbreviation='AN')
    sen = Chamber.objects.get(abbreviation='SEN')
    for data in ijson.items(stream or sys.stdin, 'item'):
        parse_dossier_data(data, an, sen)



# coding: utf-8
import logging
import sys
import urllib2

import ijson
import django
from django.apps import apps
from django.db import transaction

from representatives.models import Chamber
from representatives_votes.models import Dossier, Document
from .import_votes import Command

logger = logging.getLogger(__name__)

BASEURL = 'http://parltrack.euwiki.org/'
URL = 'http://parltrack.euwiki.org/dumps/ep_dossiers.json.xz'
LOCAL_PATH = 'ep_dossiers.json.xz'


def parse_dossier_data(data, ep):
    """Parse data from parltarck dossier export (1 dossier) Update dossier
    if it existed before, this function goal is to import and update a
    dossier, not to import all parltrack data
    """
    changed = False
    doc_changed = False
    ref = data['procedure']['reference']

    logger.debug('Processing dossier %s', ref)

    with transaction.atomic():
        try:
            dossier = Dossier.objects.get(reference=ref)
        except Dossier.DoesNotExist:
            dossier = Dossier(reference=ref)
            logger.debug('Dossier did not exist')
            changed = True

        if dossier.title != data['procedure']['title']:
            logger.debug('Title changed from "%s" to "%s"', dossier.title,
                         data['procedure']['title'])
            dossier.title = data['procedure']['title']
            changed = True

        if changed:
            logger.info('Updated dossier %s', ref)
            dossier.save()

        source = data['meta']['source'].replace('&l=en', '')
        try:
            doc = Document.objects.get(dossier=dossier, kind='procedure-file')
        except Document.DoesNotExist:
            doc = Document(dossier=dossier, kind='procedure-file', chamber=ep)
            logger.debug('Document for dossier %s did not exist', ref)
            doc_changed = True

        if doc.link != source:
            logger.debug('Link changed from %s to %s', doc.link, source)
            doc.link = source
            doc_changed = True

        if doc_changed:
            logger.info('Updated document %s for dossier %s', doc.link, ref)
            doc.save()

    if 'votes' in data.keys() and 'epref' in data['votes']:
        command = Command()
        command.init_cache()
        command.parse_vote_data(data['votes'])


def sync_dossier(reference):
    url = '%s/dossier/%s?format=json' % (BASEURL, reference)

    logger.debug('Syncing dossier from %s' % url)
    with urllib2.urlopen(url) as stream:
        import_single(stream)


def import_single(stream):
    if not apps.ready:
        django.setup()

    ep = Chamber.objects.get(abbreviation='EP')
    for data in ijson.items(stream, ''):
        parse_dossier_data(data, ep)


def main(stream=None):
    if not apps.ready:
        django.setup()

    ep = Chamber.objects.get(abbreviation='EP')
    for data in ijson.items(stream or sys.stdin, 'item'):
        parse_dossier_data(data, ep)



# coding: utf-8

from django.contrib import admin

from .models import Dossier, Document, Proposal, Vote


class DossierAdmin(admin.ModelAdmin):
    list_display = ('id', 'reference', 'title')
    search_fields = ('reference', 'title')


class DocumentAdmin(admin.ModelAdmin):
    list_display = ('dossier_reference', 'kind', 'title', 'link')
    search_fields = ('reference', 'dossier__reference', 'title')

    def dossier_reference(self, obj):
        return obj.dossier.reference


class ProposalAdmin(admin.ModelAdmin):
    list_display = (
        'reference',
        'dossier_reference',
        'title',
        'kind')
    search_fields = ('reference', 'dossier__reference', 'title')

    def dossier_reference(self, obj):
        return obj.dossier.reference


class NoneMatchingFilter(admin.SimpleListFilter):
    title = 'Representative'
    parameter_name = 'representative'

    def lookups(self, request, model_admin):
        return [('None', 'Unknown')]

    def queryset(self, request, queryset):
        if self.value() == 'None':
            return queryset.filter(representative=None)
        else:
            return queryset


class VoteAdmin(admin.ModelAdmin):
    list_display = (
        'id',
        'proposal_reference',
        'position',
        'representative',
        'representative_name')
    list_filter = (NoneMatchingFilter,)

    def proposal_reference(self, obj):
        return obj.proposal.reference

admin.site.register(Dossier, DossierAdmin)
admin.site.register(Document, DocumentAdmin)
admin.site.register(Proposal, ProposalAdmin)
admin.site.register(Vote, VoteAdmin)



# coding: utf-8

import ijson
import logging
import sys

import django
from django.apps import apps
from django.utils.text import slugify

from representatives_votes.models import Proposal, Representative, Vote

logger = logging.getLogger(__name__)


class VotesImporter:
    deputes_slug = None
    deputes_rid = None
    scrutins = None
    touched = []

    positions = dict(
        pour="for",
        contre="against",
        abstention="abstain"
    )

    def get_depute_by_name(self, prenom, nom):
        if self.deputes_slug is None:
            self.deputes_slug = {
                slugify(r[0]): r[1] for r in
                Representative.objects.values_list('full_name', 'pk')
            }

        full = (u'%s %s' % (prenom, nom)).replace(u' ', ' ')
        return self.deputes_slug.get(slugify(full), None)

    def get_depute_by_url(self, url):
        if self.deputes_rid is None:
            self.deputes_rid = {
                r[0]: r[1] for r in
                Representative.objects.prefetch_related('website_set')
                .filter(website__kind__in=['AN', 'SEN'])
                .values_list('website__url', 'pk')
            }

        return self.deputes_rid.get(url, None)

    def get_scrutin(self, ref):
        if self.scrutins is None:
            self.scrutins = {
                s[0]: s[1] for s in Proposal.objects.values_list('reference',
                                                                 'pk')
            }

        return self.scrutins.get(ref, None)

    def parse_vote_data(self, data):
        scrutin = self.get_scrutin(data['scrutin_url'])
        if scrutin is None:
            logger.debug('Cannot import vote for unknown scrutin %s'
                         % data['scrutin_url'])
            return

        if 'parl_url' in data:
            repdesc = data['parl_url']
            depute = self.get_depute_by_url(data['parl_url'])
        else:
            repdesc = '%s %s' % (data['prenom'], data['nom'])
            depute = self.get_depute_by_name(data['prenom'], data['nom'])

        if depute is None:
            logger.debug('Cannot import vote by unknown rep %s' % repdesc)
            return

        if not data['division'].lower() in self.positions:
            logger.debug('Cannot import vote for invalid position %s'
                         % data['division'])
            return
        position = self.positions[data['division'].lower()]

        changed = False
        try:
            vote = Vote.objects.get(representative_id=depute,
                                    proposal_id=scrutin)
        except Vote.DoesNotExist:
            vote = Vote(representative_id=depute, proposal_id=scrutin)
            logger.debug('Created vote for rep %s on %s' % (depute, scrutin))
            changed = True

        if vote.position != position:
            logger.debug('Changed vote position to %s' % position)
            changed = True
            vote.position = position

        if changed:
            logger.debug('Updated vote for rep %s on %s' % (depute, scrutin))
            self.touched.append(scrutin)
            vote.save()

    def update_totals(self):
        proposals = [Proposal.objects.get(pk=pk) for pk in self.touched]

        for proposal in proposals:
            changed = False

            for pos in self.positions.values():
                count = Vote.objects.filter(proposal_id=proposal.pk,
                                            position=pos).count()

                if getattr(proposal, 'total_%s' % pos, None) != count:
                    logger.debug('Changed %s count for proposal %s to %s' % (
                        pos, proposal.pk, count))
                    setattr(proposal, 'total_%s' % pos, count)
                    changed = True

            if changed:
                logger.debug('Updated proposal %s' % proposal.pk)
                proposal.save()


def main(stream=None):
    if not apps.ready:
        django.setup()

    importer = VotesImporter()

    for data in ijson.items(stream or sys.stdin, 'item'):
        importer.parse_vote_data(data)

    importer.update_totals()



import copy
import os
import pytest

from django.core.serializers.json import Deserializer
from django.core.management import call_command

from representatives.models import Representative
from representatives_votes.contrib.francedata import import_dossiers
from representatives_votes.contrib.francedata import import_scrutins
from representatives_votes.contrib.francedata import import_votes
from representatives_votes.models import Dossier, Proposal, Vote


def _get_testdata(filename):
    return os.path.join(os.path.dirname(__file__), filename)


def _test_import(fixtures, scenario, callback):
    for model in (Representative, Dossier, Proposal, Vote):
        model.objects.all().delete()

    for fix in fixtures:
        call_command('loaddata', fix)

    inputfile = _get_testdata('%s_input.json' % scenario)
    expected = _get_testdata('%s_expected.json' % scenario)

    # Disable django auto fields
    exclude = ('id', '_state', 'created', 'updated', 'fingerprint')

    with open(inputfile, 'r') as f:
        callback(f)

    with open(expected, 'r') as f:
        for obj in Deserializer(f.read()):
            compare = copy.copy(obj.object.__dict__)

            for f in exclude:
                if f in compare:
                    compare.pop(f)

            type(obj.object).objects.get(**compare)


@pytest.mark.django_db
def test_francedata_import_dossiers():
    fixtures = []

    _test_import(fixtures, 'dossiers', import_dossiers.main)


@pytest.mark.django_db
def test_francedata_import_scrutins():
    fixtures = [
        _get_testdata('dossiers_expected.json')
    ]

    _test_import(fixtures, 'scrutins', import_scrutins.main)


@pytest.mark.django_db
def test_francedata_import_votes():
    fixtures = [
        _get_testdata('dossiers_expected.json'),
        _get_testdata('scrutins_expected.json'),
        _get_testdata('rep_fixture.json')
    ]

    _test_import(fixtures, 'votes', import_votes.main)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('representatives_votes', '0003_auto_20150708_1358'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='proposal',
            options={
                'ordering': ['datetime']},
        ),
        migrations.AlterModelOptions(
            name='vote',
            options={
                'ordering': ['proposal__datetime']},
        ),
        migrations.AlterField(
            model_name='proposal',
            name='representatives',
            field=models.ManyToManyField(
                related_name='proposals',
                through='representatives_votes.Vote',
                to='representatives.Representative'),
        ),
    ]
